package com.pusong.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.pusong.business.domain.bo.PsSalaryConfigAddVo;
import com.pusong.business.domain.vo.PsSalaryConfigInfoVo;
import com.pusong.common.core.exception.ServiceException;
import com.pusong.common.core.utils.MapstructUtils;
import com.pusong.common.mybatis.core.page.TableDataInfo;
import com.pusong.common.mybatis.core.page.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import com.pusong.business.domain.bo.PsSalaryConfigBo;
import com.pusong.business.domain.vo.PsSalaryConfigVo;
import com.pusong.business.domain.PsSalaryConfig;
import com.pusong.business.mapper.PsSalaryConfigMapper;
import com.pusong.business.service.IPsSalaryConfigService;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 提成配置Service业务层处理
 *
 * @author Lion Li
 * @date 2024-08-22
 */
@RequiredArgsConstructor
@Service
public class PsSalaryConfigServiceImpl implements IPsSalaryConfigService {

    private static final Logger log = LoggerFactory.getLogger(PsSalaryConfigServiceImpl.class);
    private final PsSalaryConfigMapper baseMapper;



    /**
     * 分页查询提成配置列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 提成配置分页列表
     */
    @Override
    public TableDataInfo<PsSalaryConfigVo> queryPageList(PsSalaryConfigBo bo, PageQuery pageQuery) {
        QueryWrapper<PsSalaryConfig> lqw = buildQueryWrapper(bo);
        Page<PsSalaryConfigVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        if (CollectionUtils.isEmpty(result.getRecords())) {return TableDataInfo.build(result);}
        result.getRecords().forEach(item->{
            List<String> pro = List.of(item.getServiceProject().split(","));
            item.setServiceProject(String.join(",",pro.stream().map(str->str.split("_")[0]).collect(Collectors.toSet())));
        });
        return TableDataInfo.build(result);
    }
    private QueryWrapper<PsSalaryConfig> buildQueryWrapper(PsSalaryConfigBo bo) {
        QueryWrapper<PsSalaryConfig> lqw = Wrappers.query();
        lqw.select("template_code","max(template_name) template_name","GROUP_CONCAT(DISTINCT  service_project SEPARATOR ',') service_project"
                ,"GROUP_CONCAT(DISTINCT post_id SEPARATOR ',') post_ids","max(create_by) create_by","max(create_time) create_time");
        lqw.like(StringUtils.isNotBlank(bo.getTemplateName()),"template_name", bo.getTemplateName());
        lqw.eq(StringUtils.isNotBlank(bo.getTemplateCode()),"template_code", bo.getTemplateCode());
        lqw.groupBy("template_code");
        return lqw;
    }
    /**
     * 查询提成配置详情
     *
     * @param templateCode        模板编码
     * @return PsSalaryConfigVo
     */
    public PsSalaryConfigInfoVo queryByTemplateCode(String templateCode){
        List<PsSalaryConfig> list = baseMapper.selectList(Wrappers.<PsSalaryConfig>lambdaQuery().eq(PsSalaryConfig::getTemplateCode, templateCode).eq(PsSalaryConfig::getDelFlag, 0));
        PsSalaryConfigInfoVo vo = new PsSalaryConfigInfoVo();
        //服务类型：服务项目：成本
        //               ：提成比例
        Map<String,Map<String,Map<String,String>>> map = new HashMap<>();
        Set<String> serviceTypeList = new HashSet<>();
        Set<String> postIdList = new HashSet<>();
        list.forEach(item->{
            //服务类型
            String serviceType = item.getServiceProject().split("_")[0];
            //服务类型code拼接逗号分割
            serviceTypeList.add(serviceType);
            //岗位id拼接
            postIdList.add(item.getPostId()+"");
//            vo.setServiceType(StringUtils.isBlank(vo.getServiceType())?serviceType:vo.getServiceType()+","+serviceType);
//            //岗位id拼接
//            vo.setPostId((StringUtils.isBlank(vo.getPostId())?"":vo.getPostId()+",")+item.getPostId());
            //服务项目：提成比例
            Map<String, Map<String,String>> project = CollectionUtils.isNotEmpty(map.get(serviceType))?map.get(serviceType):new HashMap<>();
            Map<String,String> data = new HashMap<>();
            data.put("cost",item.getCost().stripTrailingZeros().toPlainString());
            data.put("salaryRatio",item.getSalaryRatio().stripTrailingZeros().toPlainString());
            project.put(item.getServiceProject(),data);
            map.put(serviceType,project);
        });
        vo.setServiceType(String.join(",",serviceTypeList));
        vo.setPostId(String.join(",",postIdList));
        vo.setServiceTypeMap(map);
        vo.setTemplateCode(templateCode);
        vo.setTemplateName(list.get(0).getTemplateName());
        return vo;
    }

    /**
     * 新增或修改提成配置详情
     *
     * @param addVo        提成模板信息
     */
    @Transactional(rollbackFor = Exception.class)
    public void add(PsSalaryConfigAddVo addVo){
        List<PsSalaryConfig> list = MapstructUtils.convert(addVo.getSalaryConfigVoList(), PsSalaryConfig.class);
        String code = addVo.getTemplateCode() == null ? UUID.randomUUID().toString() : addVo.getTemplateCode();

        if(StringUtils.isNotBlank(addVo.getTemplateCode())){//修改
            baseMapper.delete(Wrappers.<PsSalaryConfig>lambdaQuery().eq(PsSalaryConfig::getTemplateCode, code));
            log.info("修改配置成功");
        }
        for (String post : addVo.getPostId().split(",")) {
            List<String> spList = new ArrayList<>();
            list.forEach(item->{
                item.setId(null);
                item.setTemplateCode(code);
                item.setTemplateName(addVo.getTemplateName());
                item.setPostId(Long.valueOf(post));
                spList.add(item.getServiceProject());
            });
            LambdaQueryWrapper<PsSalaryConfig> wq = Wrappers.<PsSalaryConfig>lambdaQuery().eq(PsSalaryConfig::getDelFlag, 0)
                    .eq(PsSalaryConfig::getPostId,post).in(PsSalaryConfig::getServiceProject,spList);

            Long num = baseMapper.selectCount(Wrappers.<PsSalaryConfig>lambdaQuery().eq(PsSalaryConfig::getDelFlag, 0)
                    .eq(PsSalaryConfig::getPostId, post).in(PsSalaryConfig::getServiceProject, spList));
            if(num>0){
                throw new ServiceException("岗位的服务项目重复");
            }
            baseMapper.insertBatch(list);
        }
    }
    /**
     * 校验并批量删除提成配置信息
     *
     * @param code     编码
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(String code) {
        return baseMapper.delete(Wrappers.<PsSalaryConfig>lambdaQuery().eq(PsSalaryConfig::getTemplateCode,code)) > 0;
    }

//    /**
//     * 查询提成配置
//     *
//     * @param id 主键
//     * @return 提成配置
//     */
//    @Override
//    public PsSalaryConfigVo queryById(Long id){
//        return baseMapper.selectVoById(id);
//    }
//    /**
//     * 查询符合条件的提成配置列表
//     *
//     * @param bo 查询条件
//     * @return 提成配置列表
//     */
//    @Override
//    public List<PsSalaryConfigVo> queryList(PsSalaryConfigBo bo) {
//        QueryWrapper<PsSalaryConfig> lqw = buildQueryWrapper(bo);
//        return baseMapper.selectVoList(lqw);
//    }
//
//
//
//    /**
//     * 新增提成配置
//     *
//     * @param bo 提成配置
//     * @return 是否新增成功
//     */
//    @Override
//    public Boolean insertByBo(PsSalaryConfigBo bo) {
//        PsSalaryConfig add = MapstructUtils.convert(bo, PsSalaryConfig.class);
//        validEntityBeforeSave(add);
//        boolean flag = baseMapper.insert(add) > 0;
//        return flag;
//    }
//
//    /**
//     * 修改提成配置
//     *
//     * @param bo 提成配置
//     * @return 是否修改成功
//     */
//    @Override
//    public Boolean updateByBo(PsSalaryConfigBo bo) {
//        PsSalaryConfig update = MapstructUtils.convert(bo, PsSalaryConfig.class);
//        validEntityBeforeSave(update);
//        return baseMapper.updateById(update) > 0;
//    }
//
//    /**
//     * 保存前的数据校验
//     */
//    private void validEntityBeforeSave(PsSalaryConfig entity){
//        //TODO 做一些数据校验,如唯一约束
//    }
//
//    /**
//     * 校验并批量删除提成配置信息
//     *
//     * @param ids     待删除的主键集合
//     * @param isValid 是否进行有效性校验
//     * @return 是否删除成功
//     */
//    @Override
//    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
//        if(isValid){
//            //TODO 做一些业务上的校验,判断是否需要校验
//        }
//        return baseMapper.deleteBatchIds(ids) > 0;
//    }
}
